#ifndef _CHRONOPTICS_TOF_CSF_READER_H_
#define _CHRONOPTICS_TOF_CSF_READER_H_

#include <chronoptics/tof/data.h>
#include <chronoptics/tof/stream.h>
#include <chronoptics/tof/calibration.h>
#include <chronoptics/tof/camera_config.h>

#ifdef __cplusplus
extern "C" {
#endif

/** This class reads csf files
 */
typedef struct tof_csf_reader* tof_csf_reader_t;

/** Destruct tof_csf_reader */
TOF_EXPORT void tof_csf_reader_delete(tof_csf_reader_t ptr);

/** Open a csf file for reading
 * @param ptr Pointer to class
 * @param file_location File location of the csf file
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_csf_reader_t tof_csf_reader_new(const char* file_location, tof_error_t *error);

/** Read the next data frame from disk
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return A data frame
 */
TOF_EXPORT tof_data_t tof_csf_reader_read_frame(tof_csf_reader_t ptr, tof_error_t *error);

/** Read the next data frame from disk into the supplied pointer
 * @param ptr Pointer to class
 * @param pointer The pointer to the data to write into
 * @param capacity The amount of data the pointer can hold
 * @param callback Callback that will be called when pointer is no longer in
 * use
 * @param callback_user_data User data that will be passed back when the function pointer is called
 * @param error Pointer to error
 * @return A data frame
 */
TOF_EXPORT tof_data_t tof_csf_reader_read_frame_into_pointer(tof_csf_reader_t ptr, uint8_t* pointer, size_t capacity, tof_user_pointer_destructed_t callback, void* callback_user_data, tof_error_t *error);

/** Get the total number of frames in the csf file
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Total number of frames in the csf file
 */
TOF_EXPORT size_t tof_csf_reader_number_of_frames(const tof_csf_reader_t ptr, tof_error_t *error);

/** Returns an empty Data class containing the header information for the
 * specified frame
 * @param ptr Pointer to class
 * @param index Which frame to get the header from
 * @param error Pointer to error
 * @return Empty Data instance containing header information
 */
TOF_EXPORT tof_data_t tof_csf_reader_get_header(const tof_csf_reader_t ptr, size_t index, tof_error_t *error);

/** Get the stream information of the next frame
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Information on the next frame
 */
TOF_EXPORT tof_stream_t tof_csf_reader_next_stream(const tof_csf_reader_t ptr, tof_error_t *error);

/** Get the size of the next frame
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Size of the next frame
 */
TOF_EXPORT size_t tof_csf_reader_next_frame_size(const tof_csf_reader_t ptr, tof_error_t *error);

/** Get the current frame position
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Frame index
 */
TOF_EXPORT size_t tof_csf_reader_frame_index(const tof_csf_reader_t ptr, tof_error_t *error);

/** Go to specified frame
 * @param ptr Pointer to class
 * @param index The index of the frame to jump to
 * @param error Pointer to error
 */
TOF_EXPORT void tof_csf_reader_go_to_frame(tof_csf_reader_t ptr, size_t index, tof_error_t *error);

/** Get the streams that are in the csf file
 * @param ptr Pointer to class
 * @param streams Streams in the csf file
 * @param capacity The amount of data the above pointer can hold
 * @param error Pointer to error
 * @return Number of pointers filled or complete array size
 */
TOF_EXPORT size_t tof_csf_reader_streams(const tof_csf_reader_t ptr, tof_stream_t* streams, size_t capacity, tof_error_t *error);

/** Get the number of frames contained in the file of the given stream
 * @param ptr Pointer to class
 * @param stream The stream
 * @param error Pointer to error
 * @return Number of frames for given stream
 */
TOF_EXPORT size_t tof_csf_reader_stream_count(const tof_csf_reader_t ptr, tof_stream_t stream, tof_error_t *error);

/** Go to the specified stream, the next read frame call will start from here
 * @param ptr Pointer to class
 * @param stream The stream
 * @param index The frame index
 * @param error Pointer to error
 */
TOF_EXPORT void tof_csf_reader_go_to_stream(tof_csf_reader_t ptr, tof_stream_t stream, size_t index, tof_error_t *error);

/** Get calibration from the csf file
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Calibration
 */
TOF_EXPORT tof_calibration_t tof_csf_reader_get_calibration(const tof_csf_reader_t ptr, tof_error_t *error);

/** Get camera config from the csf file
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Camera config
 */
TOF_EXPORT tof_camera_config_t tof_csf_reader_get_camera_config(const tof_csf_reader_t ptr, tof_error_t *error);

#ifdef __cplusplus
}
#endif

#endif
